Index: pbx.c
===================================================================
--- pbx.c	(revision 7815)
+++ pbx.c	(working copy)
@@ -433,13 +433,14 @@
 
 	{ "WaitExten", pbx_builtin_waitexten, 
 	"Waits for an extension to be entered", 
-	"  WaitExten([seconds][|options]): This application waits for the user to enter\n"
+	"  WaitExten([seconds][|options][|maxtimeout]): This application waits for the user to enter\n"
 	"a new extension for a specified number of seconds.\n"
 	"  Note that the seconds can be passed with fractions of a second. For example,\n"
 	"'1.5' will ask the application to wait for 1.5 seconds.\n" 
 	"  Options:\n"
 	"    m[(x)] - Provide music on hold to the caller while waiting for an extension.\n"
 	"               Optionally, specify the class for music on hold within parenthesis.\n"
+	"  the channel will hangup after maxtimeout times timeout, maxtimeout default to 3\n"
 	},
 
 };
@@ -5496,40 +5497,53 @@
  */
 static int pbx_builtin_waitexten(struct ast_channel *chan, void *data)
 {
-	int ms, res, argc;
-	char *args;
-	char *argv[2];
-	char *options = NULL; 
-	char *timeout = NULL;
+	int ms, res, maxtimeout;
 	struct ast_flags flags = {0};
 	char *opts[1] = { NULL };
+	char *parse;
+	AST_DECLARE_APP_ARGS(args,
+		AST_APP_ARG(timeout);
+		AST_APP_ARG(options);
+		AST_APP_ARG(maxtimeout);
+	);
 
-	args = ast_strdupa(data);
-
-	if ((argc = ast_app_separate_args(args, '|', argv, sizeof(argv) / sizeof(argv[0])))) {
-		if (argc > 0) {
-			timeout = argv[0];
-			if (argc > 1)
-				options = argv[1];
+	if (!ast_strlen_zero((char *)data)) {
+		parse = ast_strdupa(data);
+		if (!parse) {
+			ast_log(LOG_ERROR, "Out of memory!\n");
+			return -1;
 		}
-	}
+		AST_STANDARD_APP_ARGS(args, parse);
+	} else
+		memset(&args, 0, sizeof(args));
 
-	if (options)
-		ast_app_parse_options(waitexten_opts, &flags, opts, options);
+	if (args.options)
+		ast_app_parse_options(waitexten_opts, &flags, opts, args.options);
 	
 	if (ast_test_flag(&flags, WAITEXTEN_MOH))
 		ast_moh_start(chan, opts[0]);
 
 	/* Wait for "n" seconds */
-	if (timeout && atof((char *)timeout)) 
-		ms = atof((char *)timeout) * 1000;
+	if (args.timeout && atof((char *)args.timeout)) 
+		ms = atof((char *)args.timeout) * 1000;
 	else if (chan->pbx)
 		ms = chan->pbx->rtimeout * 1000;
 	else
 		ms = 10000;
+
+	if (args.maxtimeout && atoi((char*)args.maxtimeout))
+		maxtimeout = atoi((char*)args.maxtimeout);
+	else
+		maxtimeout = 3;
+
 	res = ast_waitfordigit(chan, ms);
 	if (!res) {
-		if (ast_exists_extension(chan, chan->context, chan->exten, chan->priority + 1, chan->cid.cid_num)) {
+		chan->maxtimeout += 1;
+		if(chan->maxtimeout >= maxtimeout)
+		{
+			ast_log(LOG_WARNING, "max Timeout %d times, hangup\n", chan->maxtimeout);
+			res = -1;
+		} else if (ast_exists_extension(chan, chan->context, chan->exten, chan->priority + 1, chan->cid.cid_num)) {
 			if (option_verbose > 2)
 				ast_verbose(VERBOSE_PREFIX_3 "Timeout on %s, continuing...\n", chan->name);
 		} else if (ast_exists_extension(chan, chan->context, "t", 1, chan->cid.cid_num)) {
@@ -5541,7 +5555,8 @@
 			ast_log(LOG_WARNING, "Timeout but no rule 't' in context '%s'\n", chan->context);
 			res = -1;
 		}
-	}
+	} else
+		chan->maxtimeout = 0;
 
 	if (ast_test_flag(&flags, WAITEXTEN_MOH))
 		ast_moh_stop(chan);
